package com.usky.iot.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.PmProject;
import com.usky.iot.domain.PmWorkContent;
import com.usky.iot.domain.PmWorkReport;
import com.usky.iot.mapper.PmProjectMapper;
import com.usky.iot.mapper.PmWorkContentMapper;
import com.usky.iot.service.PmProjectService;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 项目表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-05-20
 */
@Service
public class PmProjectServiceImpl extends AbstractCrudService<PmProjectMapper, PmProject> implements PmProjectService {

    @Autowired
    private PmWorkContentMapper pmWorkContentMapper;

    @Override
    public void addProject(PmProject project) {
        if (Objects.isNull(project.getProjectWorkload())) {
            if (Objects.isNull(project.getId())) {
                project.setCreateBy(SecurityUtils.getUsername());
                project.setCreateTime(LocalDateTime.now());
                project.setTenantId(SecurityUtils.getTenantId());
                project.setDeptId(SecurityUtils.getLoginUser().getSysUser().getDeptId());
                this.save(project);
            } else {
                project.setUpdateBy(SecurityUtils.getUsername());
                project.setUpdateTime(LocalDateTime.now());
                this.updateById(project);
            }
        } else {
            if (project.getProjectWorkload().scale() > 2) {
                throw new BusinessException("计划人/天小数位超出长度请重新输入");
            } else if (project.getProjectWorkload().precision() - project.getProjectWorkload().scale() > 4) {
                throw new BusinessException("计划人/天整数位超出长度请重新输入");
            } else {
                if (Objects.isNull(project.getId())) {
                    project.setCreateBy(SecurityUtils.getUsername());
                    project.setCreateTime(LocalDateTime.now());
                    project.setTenantId(SecurityUtils.getTenantId());
                    project.setDeptId(SecurityUtils.getLoginUser().getSysUser().getDeptId());
                    this.save(project);
                } else {
                    project.setUpdateBy(SecurityUtils.getUsername());
                    project.setUpdateTime(LocalDateTime.now());
                    this.updateById(project);
                }
            }
        }
    }

    @Override
    public void delProject(Integer projectId) {
        LambdaQueryWrapper<PmWorkContent> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PmWorkContent::getProjectId, projectId);
        List<PmWorkContent> pmWorkContent = pmWorkContentMapper.selectList(queryWrapper);
        if (pmWorkContent.size() != 0) {
            throw new BusinessException("该项目下存在日报，无法删除！");
        } else {
            LambdaQueryWrapper<PmProject> query = Wrappers.lambdaQuery();
            query.eq(PmProject::getId, projectId);
            PmProject project = baseMapper.selectOne(query);
            project.setDelFlag(1);
            project.setUpdateBy(SecurityUtils.getUsername());
            project.setUpdateTime(LocalDateTime.now());
            baseMapper.updateById(project);
        }
    }

    /**
     * 添加工作报告下拉框
     * 查询当前租户所有项目
     *
     * @return 只返回项目id和名称
     */
    @Override
    public List<PmProject> queryProject() {
        LambdaQueryWrapper<PmProject> wrapper = Wrappers.lambdaQuery();
        wrapper.select(PmProject::getId, PmProject::getProjectName)
                .eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .and(qw -> qw
                        .eq(PmProject::getProjectHead, SecurityUtils.getUserId())
                        .or().apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0")
                )
                .in(PmProject::getProjectStatus, 1, 2, 3);
        List<PmProject> list = this.list(wrapper);
        return list;
    }

    /**
     * 分页
     *
     * @param projectName       项目名
     * @param projectType       类型
     * @param projectStatus     状态
     * @param projectAscription 项目归属类型
     * @param pageNum           页码
     * @param pageSize          页大小
     * @return
     */
    @Override
    public CommonPage<PmProject> projectList(String projectName, Integer projectType, Integer projectStatus, Integer projectAscription, Integer pageNum, Integer pageSize) {
        IPage<PmProject> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<PmProject> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .orderByDesc(PmProject::getCreateTime);
        if (projectAscription == 0) {
            lambdaQuery.and(q -> q.eq(PmProject::getProjectHead, SecurityUtils.getUserId())
                            .or().apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0")
                            .or().eq(PmProject::getCreateBy, SecurityUtils.getUsername()));
        } else if (projectAscription == 1) {
            lambdaQuery.eq(PmProject::getProjectHead, SecurityUtils.getUserId());
        } else if (projectAscription == 2) {
            lambdaQuery.apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0");
        }
        if (!projectName.isEmpty()) {
            lambdaQuery.like(PmProject::getProjectName, projectName);
        }
        if (projectType != 0) {
            lambdaQuery.eq(PmProject::getProjectType, projectType);
        }
        if (projectStatus != 0) {
            lambdaQuery.eq(PmProject::getProjectStatus, projectStatus);
        }
        page = this.page(page, lambdaQuery);
        if (page.getRecords() == null || page.getRecords().isEmpty()) {
            return new CommonPage<>(Collections.emptyList(), 0, pageSize, pageNum);
        }
        return new CommonPage<>(page.getRecords(), page.getTotal(), pageSize, pageNum);
    }

    /**
     * 查询项目名
     *
     * @param projectIds
     * @return
     */
    @Override
    public List<PmProject> projectName(List<Integer> projectIds) {
        LambdaQueryWrapper<PmProject> wrapper = Wrappers.lambdaQuery();
        wrapper.select(PmProject::getId, PmProject::getProjectName)
                .in(PmProject::getId, projectIds);
        return this.list(wrapper);
    }

    /**
     * 统计
     *
     * @return
     */
    @Override
    public Map<String, Integer> sum() {
        LambdaQueryWrapper<PmProject> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .and(q -> q.eq(PmProject::getProjectHead, SecurityUtils.getUserId())
                        .or().apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0")
                .or().eq(PmProject::getCreateBy, SecurityUtils.getUsername()))
                .eq(PmProject::getDelFlag, 0);
        Integer all = this.count(lambdaQuery);

        LambdaQueryWrapper<PmProject> lambdaQuery1 = Wrappers.lambdaQuery();
        lambdaQuery1.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .eq(PmProject::getProjectHead, SecurityUtils.getUserId());
        Integer head = this.count(lambdaQuery1);

        LambdaQueryWrapper<PmProject> lambdaQuery2 = Wrappers.lambdaQuery();
        lambdaQuery2.eq(PmProject::getTenantId, SecurityUtils.getTenantId())
                .eq(PmProject::getDelFlag, 0)
                .apply("FIND_IN_SET('" + SecurityUtils.getUserId() + "', project_member) > 0");
        Integer join = this.count(lambdaQuery2);

        Map<String, Integer> sumMap = new HashMap<>();
        sumMap.put("allProject", all);
        sumMap.put("headProject", head);
        sumMap.put("joinProject", join);
        return sumMap;
    }

}
